/****************************************************************************
*  This file is part of the speed test of ChibiOS/RT
*
*  Copyright (c) 2011 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  19.02.2011  mifi   First Version, tested with a AT91SAM7X-EK board
****************************************************************************/
#define __MAIN_C__

#include <ch.h>
#include <targets/at91sam7.h>

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
/*
 * The LED1 is connected to port PB0.
 * If the LED on your eval board is connected to an 
 * other port, you must change the defines here.
 */
#define LED1      (1<<0)

#define LED_PER   PIOB_PER
#define LED_OER   PIOB_OER
#define LED_IDR   PIOB_IDR

#define LED_ON    PIOB_CODR
#define LED_OFF   PIOB_SODR

/*
 * Priority and stack size of the task.
 */
#define TASK_HP_PRIORITY   NORMALPRIO + 3
#define TASK_LP_PRIORITY   NORMALPRIO + 2

#define TASK_HP_STK_SIZE   64
#define TASK_LP_STK_SIZE   64

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/
static unsigned long *SourceModeRegisterTable   = (unsigned long*)(AIC_BASE + AIC_SMR0_OFFSET);
static unsigned long *SourceVectorRegisterTable = (unsigned long*)(AIC_BASE + AIC_SVR0_OFFSET);

static WORKING_AREA(HPStack, TASK_HP_STK_SIZE);
static WORKING_AREA(LPStack, TASK_LP_STK_SIZE);

static Mailbox mb1;
static msg_t   MBuffer[8];

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/
/***************************************************************************/
/*  LedOn                                                                  */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LedOn (void)
{
   LED_ON = LED1;
} /* LedOn */

/***************************************************************************/
/*  LedOff                                                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LedOff (void)
{
   LED_OFF = LED1;
} /* LedOff */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   LED_PER = LED1;
   LED_OER = LED1;
   LED_IDR = LED1;

   LedOff();
} /* LEDInit */

/***************************************************************************/
/*  SpuriousHandler                                                        */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static CH_IRQ_HANDLER (SpuriousHandler)
{
   CH_IRQ_PROLOGUE();
   AIC_EOICR = 0;
   CH_IRQ_EPILOGUE();
} /* SpuriousHandler */

/***************************************************************************/
/*  SYSIrqHandler                                                          */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static CH_IRQ_HANDLER (SYSIrqHandler) 
{
   CH_IRQ_PROLOGUE();

   if (PIT_SR & PIT_SR_PITS_MASK) 
   {
      (void)PIT_PIVR;
      chSysLockFromIsr();
      chSysTimerHandlerI();
      chSysUnlockFromIsr();
   }
   AIC_EOICR = 0;

   CH_IRQ_EPILOGUE();
} /* SYSIrqHandler */

/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void HardwareInit (void)
{
   int i;

   /* Disable all interrupts first */
   AIC_IDCR = 0xFFFFFFFF;
   
   /* Set end of interrupt command */
   AIC_EOICR = 0;

   /* Set spurious interrupt handler */ 
   AIC_SPU = (unsigned long)SpuriousHandler;
   
   /*
    * The CPU clock is set to 48MHz by the startup code
    * from CrossWorks for ARM. Here we must only init the
    * timer. Setup the Periodic Interval Timer with a period
    * of 1ms.
    */
   PMC_PCER |= (1<<2);        /* Enable PIO clock */
   PIT_MR    = (1 << 24) |    /* PIT Enable */
               (1 << 25) |    /* PIT Interrupt Enable */
               (3000*1-1);    /* Periodic Interval Value - 1 ms */
               
   /*
    * Install the SystemIRQHandler, which will be used
    * for the timer interrupt.
    *
    * Setup System Interrupt Mode and Vector with Priority 7
    * PIT's int is System Interrupt, Peripheral ID for PIT is 1
    */
   SourceModeRegisterTable[1]   = (unsigned long)((1 << 5) | 7);
   SourceVectorRegisterTable[1] = (unsigned long)SYSIrqHandler;
               
   /*
    * Enable System Peripheral Int
    */
   AIC_IECR = (1<<1);
   
   /*
    * Init the GPIO for the LED
    */
   LEDInit();

} /* HardwareInit */

/***************************************************************************/
/*  HPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t HPTask (void *p)
{
   msg_t HPmessage;
   p = p;
   
   while(1)
   {
      chMBFetch(&mb1, &HPmessage, TIME_INFINITE);
      LedOff();
   }
} /* HPTask */

/***************************************************************************/
/*  LPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t LPTask (void *p)
{
   msg_t LPmessage = 42;
   
   p = p;
   
   while(1)
   {
      chThdSleepMilliseconds(100);
      
      LedOn();
      LedOff();
      
      LedOn();
      chMBPost(&mb1, LPmessage, TIME_INFINITE);
   }
} /* LPTask */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  main                                                                   */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
int main (void)
{
   /*
    * Before the application which is using ChibiOS/RT can be started, 
    * the hardware of the target must be initialized. The CPU is still
    * initialized by the CrossWorks for ARM startup. But the timer tick
    * must be init too. Even the GPIO can be init here.
    */
   HardwareInit();

   /*
    * ChibiOS/RT initialization.
    */
   chSysInit();
      
   /*
    * Init the mailbox
    */
   chMBInit(&mb1, MBuffer, 8);
   
   /*
    * Create our threads
    */
   chThdCreateStatic(HPStack, sizeof(HPStack), TASK_HP_PRIORITY, HPTask, NULL);
   chThdCreateStatic(LPStack, sizeof(LPStack), TASK_LP_PRIORITY, LPTask, NULL);
   
   /*
    * Normal main() thread activity.
    */
   while(1) 
   {
      chThdSleepMilliseconds(500);
   }

   /*
    * This return here make no sense. But to prevent the compiler warning:
    * "return type of 'main' is not 'int'
    * An int as return is used :-)
    */ 
   return(0);
} /* main */


/*** EOF ***/
